@@ -5,8 +5,10 @@ from discord import Emoji
 from discord.errors import InvalidArgument
 
 import inspect
+from enum import IntEnum
 from typing import Any, List, Union
 
+
 class SelectOption():
     """
     An option for a select menu
@@ -22,7 +24,7 @@ class SelectOption():
     emoji : :class:`discord.Emoji` | :class:`str`, optional
         Emoji appearing before the label; default MISSING
     """
-    def __init__(self, value, label="\u200b", description=MISSING, emoji=MISSING) -> None:
+    def __init__(self, value, label="\u200b", description=None, emoji=None) -> None:
         """
         Creates a new SelectOption
 
@@ -31,14 +33,21 @@ class SelectOption():
         SelectOption(label="This is a option", value="my_value", description="This is the description of the option")
         ```
         """
-        self._json = {}
-        
+        self._label = None
+        self._value = None
+        self._description = None
+        self._emoji = None
+        self.default: bool = False
+        """
+        Whether this option is selected by default in the menu or not
+
+        :type: :class:`bool`
+        """
+
         self.label = label
         self.value = value
-        if description is not MISSING:
-            self.description = description
-        if emoji is not MISSING:
-            self.emoji = emoji
+        self.description = description
+        self.emoji = emoji
     def __repr__(self) -> str:
         return f"<discord_ui.SelectOption(label={self.label}, value={self.value})>"
 
@@ -58,7 +67,7 @@ class SelectOption():
 
         :type: :class:`str`
         """
-        return self._json["label"]
+        return self._label
     @label.setter
     def label(self, value: str):
         if value is None:
@@ -67,7 +76,7 @@ class SelectOption():
             raise WrongType("label", value, "str")
         elif value is not None and len(value) > 100 and value > 0:
             raise InvalidLength("label", value, 100, 0)
-        self._json["label"] = value
+        self._label = value
 
     @property
     def value(self) -> Any:
@@ -76,7 +85,7 @@ class SelectOption():
 
         :type: :class:`str`
         """
-        return self._json["value"]
+        return self._value
     @value.setter
     def value(self, value):
         if inspect.isclass(value):
@@ -84,7 +93,7 @@ class SelectOption():
         if isinstance(value, str):
             if len(value) > 100 or len(value) < 1:
                 raise InvalidLength("value", _min=1, _max=100)
-        self._json["value"] = value
+        self._value = value
 
     @property
     def description(self) -> str:
@@ -93,25 +102,15 @@ class SelectOption():
 
         :type: :class:`str`
         """
-        return self._json.get("description")
+        return self._description
     @description.setter
     def description(self, value):
-        if value is not MISSING and not isinstance(value, str):
+        if value is not None and not isinstance(value, str):
             raise WrongType("description", "str")
-        if value is not MISSING and len(value) > 100:
+        if value is not None and len(value) > 100:
             raise InvalidLength("description", 100, 0)
-        self._json["description"] = value
+        self._description = value
     
-    @property
-    def default(self) -> bool:
-        """
-        Whether the button is selected by default in the menu or not
-
-        :type: :class:`bool`
-        """
-        return self._json.get("default", False)
-
-
     @property
     def emoji(self) -> str:
         """
@@ -122,33 +121,46 @@ class SelectOption():
         
         :type: :class:`str`
         """
-        if "emoji" not in self._json:
+        if self._emoji is None:
             return None
-        if "id" not in self._json["emoji"]:
-            return self._json["emoji"]["name"]
-        return f'<{"a" if "animated" in self._json["emoji"] else ""}:{self._json["emoji"]["name"]}:{self._json["emoji"]["id"]}>'
+        if "id" not in self._emoji:
+            return self._emoji["name"]
+        return f'<{"a" if "animated" in self._emoji else ""}:{self._emoji["name"]}:{self._emoji["id"]}>'
     @emoji.setter
     def emoji(self, val: Union[Emoji, str, dict]):
         """The emoji appearing before the label"""
-        if isinstance(val, str):
-            self._json["emoji"] = {
+        if val is None:
+            self._emoji = None
+        elif isinstance(val, str):
+            self._emoji = {
                 "id": None,
                 "name": val
             }
         elif isinstance(val, Emoji):
-            self._json["emoji"] = {
+            self._emoji = {
                 "id": val.id,
                 "name": val.name,
                 "animated": val.animated
             }
         elif isinstance(val, dict):
-            self._json["emoji"] = val
+            self._emoji = val
+        elif val is None:
+            self._emoji = None
         else:
             raise WrongType("emoji", val, ["str", "discord.Emoji", "dict"])
 
 
     def to_dict(self) -> dict:
-        return self._json
+        payload = {
+            "label": self._label,
+            "value": self._value,
+            "default": self.default
+        }
+        if self._description is not None:
+            payload["description"] = self._description
+        if self._emoji is not None:
+            payload["emoji"] = self._emoji
+        return payload
 
     @classmethod
     def _fromData(cls, data) -> "SelectOption":
@@ -165,13 +177,13 @@ class SelectOption():
                 The new Option generated from the dict
         
         """
-        x = SelectOption("EMPTY", "EMPTY")
-        x._json = data
+        x = SelectOption(data["value"], data.get("label"), data.get("description"), data.get("emoji"))
+        x.default = data.get("default", False)
         return x
 
 class Component():
     def __init__(self, component_type) -> None:
-        self._json = {"type": component_type}
+        self._component_type = getattr(component_type, "value", component_type)
     @property
     def component_type(self) -> int:
         """
@@ -179,7 +191,12 @@ class Component():
 
         :type: :class:`int`
         """
-        return self._json["type"]
+        return ComponentType(self._component_type)
+
+class UseableComponent(Component):
+    def __init__(self, component_type) -> None:
+        Component.__init__(self, component_type)
+        self._custom_id = None
     @property
     def custom_id(self) -> str:
         """
@@ -187,17 +204,16 @@ class Component():
 
         :type: :class:`str`
         """
-        return self._json["custom_id"]
+        return self._custom_id
     @custom_id.setter
     def custom_id(self, value: str):
         if len(value) > 100 or len(value) < 1:
             raise InvalidLength("custom_id", 0, 100)
         if not isinstance(value, str):
             raise WrongType("custom_id", value, "str")
-        self._json["custom_id"] = value
+        self._custom_id = value
 
-
-class SelectMenu(Component):
+class SelectMenu(UseableComponent):
     """
     Represents a ui-dropdown selectmenu
 
@@ -218,7 +234,7 @@ class SelectMenu(Component):
     disabled: :class:`bool`, optional
         Whether the select menu should be disabled or not; default ``False``
     """
-    def __init__(self, custom_id, options, min_values = 1, max_values = 1, placeholder=MISSING, default=MISSING, disabled=False) -> None:
+    def __init__(self, custom_id, options, min_values=1, max_values=1, placeholder=None, default=None, disabled=False) -> None:
         """
         Creates a new ui select menu
 
@@ -227,75 +243,83 @@ class SelectMenu(Component):
         SelectMenu(custom_id="my_id", options=[SelectOption(...)], min_values=2, placeholder="select something", default=0)
         ```
         """
-        Component.__init__(self, ComponentType.SELECT_MENU)
+        UseableComponent.__init__(self, ComponentType.Select)
+        self._options = None
+
+        self.max_values: int = 0
+        """
+        The maximum number of items that can be chosen; default 1, max 25
+
+        :type: :class:`int`
+        """
+        self.min_values: int = 0
+        """
+        The minimum number of items that must be chosen; default 1, min 0, max 25
+
+        :type: :class:`int`
+        """
+        self.disabled = disabled
+        """
+        Whether the selectmenu is disabled or not
+
+        :type: :class:`bool`
+        """
+        self.placeholder: str = placeholder
+        """
+        Custom placeholder text if nothing is selected
+
+        :type: :class:`str` | :class:`None`
+        """
         self.custom_id = custom_id
         self.disabled = disabled
         self.options = options
 
-        if placeholder is not MISSING:
-            self.placeholder = placeholder
-
-        if min_values is not MISSING and max_values is MISSING:
+        if min_values is not None and max_values is None:
             if min_values < 0 or min_values > 25:
-                raise OutOfValidRange("min_values", 1, 25)
+                raise OutOfValidRange("min_values", 0, 25)
             self.min_values = min_values
             self.max_values = min_values
-        elif min_values is MISSING and max_values is not MISSING:
+        elif min_values is None and max_values is not None:
             if max_values < 0 or min_values > 25:
-                raise OutOfValidRange("min_values", 1, 25)
+                raise OutOfValidRange("min_values", 0, 25)
             self.min_values = 1
             self.max_values = max_values
-        elif min_values is not MISSING and max_values is not MISSING:
+        elif min_values is not None and max_values is not None:
             if max_values < 0 or min_values > 25:
-                raise OutOfValidRange("min_values", 1, 25)
+                raise OutOfValidRange("min_values", 0, 25)
             if min_values < 0 or min_values > 25:
-                raise OutOfValidRange("min_values", 1, 25)
+                raise OutOfValidRange("min_values", 0, 25)
             self.min_values = min_values
             self.max_values = max_values
         
-        if default is not MISSING:
+        if default is not None:
             self.set_default_option(default)
     def __str__(self) -> str:
         return self.custom_id
     def __repr__(self) -> str:
         return f"<discord_ui.SelectMenu(custom_id={self.custom_id}, options={self.options})>"
     
-    @classmethod
-    def _fromData(cls, data) -> 'SelectMenu':
-        x = cls._empty()
-        x._json = data
-        return x
-
     @staticmethod
-    def _empty() -> 'SelectMenu':
-        """
-        Creates a new "empty" select menu
-        
-        Returns
-        -------
-            :class:`~SelectMenu`
-                An "empty" SelectMenu
-        """
-        return SelectMenu("empty", [SelectOption("EMPTY", "EMPTY", "EMPTY")], 0, 0)
-
+    def _fromData(data) -> 'SelectMenu':
+        return SelectMenu(data["custom_id"], data["options"], data.get("min_values"), data.get("max_values"), data.get("placeholder"), disabled=data.get("disabled", False))
     # region props
     @property
     def options(self) -> List[SelectOption]:
         """
-        The choices in the select menu to select from
+        The options in the select menu to select from
 
         :type: List[:class:`~SelectOption`]
         """
-        return [SelectOption._fromData(x) for x in self._json["options"]]
+        return [SelectOption._fromData(x) for x in self._options]
     @options.setter
     def options(self, value: List[SelectOption]):
         if isinstance(value, list):
             if len(value) > 25 or len(value) == 0:
                 raise OutOfValidRange("length of options", 1, 25)
             if all(isinstance(x, SelectOption) for x in value):
-                self._json["options"] = [x.to_dict() for x in value]
+                self._options = [x.to_dict() for x in value]
             elif all(isinstance(x, dict) for x in value):
-                self._json["options"] = value
+                self._options = value
             else:
                 raise WrongType("options", value, ["List[SelectOption]", "List[dict]"])
         else:
@@ -324,72 +348,132 @@ class SelectMenu(Component):
             raise WrongType("position", position, "int")
         if position < 0 or position >= len(self.options):
             raise OutOfValidRange("default option position", 0, str(len(self.options) - 1))
-        self._json["options"][position]["default"] = True
+        self._options[position]["default"] = True
         return self
-    
-    @property
-    def placeholder(self) -> Union[str, None]:
-        """
-        Custom placeholder text if nothing is selected
-
-        :type: :class:`str` | :class:`None`
-        """
-        return self._json.get("placeholder")
-    @placeholder.setter
-    def placeholder(self, value: str):
-        self._json["placeholder"] = value
-
-    @property
-    def min_values(self) -> int:
-        """
-        The minimum number of items that must be chosen; default 1, min 0, max 25
-
-        :type: :class:`int`
-        """
-        return self._json["min_values"]
-    @min_values.setter
-    def min_values(self, value: int):
-        self._json["min_values"] = value
-
-    @property
-    def max_values(self) -> int:
-        """
-        The maximum number of items that can be chosen; default 1, max 25
-
-        :type: :class:`int`
-        """
-        return self._json["max_values"]
-    @max_values.setter
-    def max_values(self, value: int):
-        self._json["max_values"] = value
-    
-    @property
-    def disabled(self) -> bool:
-        """
-        Whether the selectmenu is disabled or not
-
-        :type: :class:`bool`
-        """
-        return self._json.get('disabled', False)
-    @disabled.setter
-    def disabled(self, value):
-        self._json["disabled"] = value
-
-    @property
-    def hash(self) -> str:
-        """
-        Hash for the selectmenu
-
-        :type: :class:`str`
-        """
-        return self._json.get("hash")
     # endregion
 
     def to_dict(self) -> dict:
-        return self._json
+        payload = {
+            "type": self._component_type,
+            "custom_id": self._custom_id,
+            "options": self._options,
+            "disabled": self.disabled,
+            "min_values": self.min_values,
+            "max_values": self.max_values
+        }
+        if self.placeholder is not None:
+            payload["placeholder"] = self.placeholder
+        return payload
 
-# region Button
-class Button(Component):
+class BaseButton():
+    def __init__(self, label, color, emoji, new_line, disabled) -> None:
+        Component.__init__(self, ComponentType.Button)
+        if label is None and emoji is None:
+            raise InvalidArgument("You need to pass a label or an emoji")
+        self._label = None
+        self._style = None
+        self._emoji = None
+        self._url = None
+
+        self.new_line = new_line
+        self.label = label
+        self.color = color
+        self.disabled = disabled
+        self.emoji = emoji
+
+    def __str__(self) -> str:
+        return self.content
+    def to_dict(self):
+        payload = {"type": self._component_type, "style": self._style, "disabled": self.disabled, "emoji": self._emoji}
+        if self._style == ButtonStyles.url:
+            payload["url"] = self._url
+        else:
+            payload["custom_id"] = self._custom_id
+        if self._emoji is not None:
+            payload["emoji"] = self._emoji
+        if self._label is not None:
+            payload["label"] = self._label
+        return payload
+
+    @property
+    def content(self) -> str:
+        """
+        The complete content in the button ("{emoji} {label}")
+
+        :type: :class:`str`
+        """
+        return (self.emoji + " " if self.emoji is not None else "") + (self.label or '')
+        
+    @property
+    def label(self) -> str:
+        """
+        The label displayed on the button
+
+        :type: :class:`str`
+        """
+        return self._label
+    @label.setter
+    def label(self, val: str):
+        if val is None:
+            val = ""
+        elif val is not None and not isinstance(val, str):
+            raise WrongType("label", val, "str")
+        elif val is not None and len(val) > 100:
+            raise InvalidLength("label", _max=100)
+        elif val is not None and len(val) < 1:
+            raise InvalidLength("label", _min=0)
+
+        self._label = str(val)
+
+    @property
+    def color(self) -> int:
+        """
+        The color for the button
+
+        :type: :class:`int`, one of :class:`~ButtonStyles`
+        """
+        return self._style
+    @color.setter
+    def color(self, val):
+        if ButtonStyles.getColor(val) is None:
+            raise InvalidArgument(str(val) + " is not a valid color")
+        self._style = ButtonStyles.getColor(val).value
+    
+    @property
+    def emoji(self) -> str:
+        """
+        The mention of the emoji before the text
+        
+            .. note::
+                For setting the emoji, you can use a str or discord.Emoji          
+        
+        :type: :class:`str`
+        """
+        if self._emoji is None:
+            return None
+        if "id" not in self._emoji:
+            return self._emoji["name"]
+        return f'<{"a" if "animated" in self._emoji else ""}:{self._emoji["name"]}:{self._emoji["id"]}>'
+    @emoji.setter
+    def emoji(self, val: Union[Emoji, str, dict]):
+        if val is None:
+            self._emoji = None
+        elif isinstance(val, str):
+            self._emoji = {
+                "name": val
+            }
+        elif isinstance(val, Emoji):
+            self._emoji = {
+                "id": val.id,
+                "name": val.name,
+                "animated": val.animated
+            }
+        elif isinstance(val, dict):
+            self._emoji = val
+        else:
+            raise WrongType("emoji", val, ["str", "discord.Emoji", "dict"])
+
+class Button(BaseButton, UseableComponent):
     """
     A ui-button
 
@@ -416,7 +500,7 @@ class Button(Component):
     disabled: :class:`bool`, optional
         Whether the button is disabled; default False
     """
-    def __init__(self, custom_id, label="\u200b", color = "blurple", emoji=MISSING, new_line=False, disabled=False) -> None:
+    def __init__(self, custom_id, label="\u200b", color = "blurple", emoji=None, new_line=False, disabled=False) -> None:
         """
         Creates a new ui-button
 
@@ -425,132 +509,14 @@ class Button(Component):
         Button("my_custom_id", "This is a cool button", "green", new_line=True)
         ```
         """
-        Component.__init__(self, ComponentType.BUTTON)
-        if label is MISSING and emoji is MISSING:
-            raise InvalidArgument("You need to pass a label or an emoji")
-        
+        BaseButton.__init__(self, label, color, emoji, new_line, disabled)
+        UseableComponent.__init__(self, self.component_type)
         self.custom_id = custom_id
-        self.new_line = new_line
-        self.label = label
-        self.color = color
-        if emoji is not MISSING:
-            self.emoji = emoji
-        self.disabled = disabled
-    def __str__(self) -> str:
-        return self.content
     def __repr__(self) -> str:
         return f"<discord_ui.Button({self.custom_id}:{self.content})>"
+    def copy(self) -> 'Button':
+        return Button(self.custom_id, self.label, self.color, self.emoji, self.new_line, self.disabled)
 
-    def to_dict(self):
-        return self._json
-
-    # region props
-    @property
-    def content(self) -> str:
-        """
-        The complete content in the button ("{emoji} {label}")
-
-        :type: :class:`str`
-        """
-        return (self.emoji + " " if self.emoji is not None else "") + (self.label or '')
-        
-    @property
-    def label(self) -> str:
-        """
-        The label displayed on the button
-
-        :type: :class:`str`
-        """
-        return self._json.get("label", None)
-    @label.setter
-    def label(self, val: str):
-        if val is None:
-            val = ""
-        elif val is not None and not isinstance(val, str):
-            raise WrongType("label", val, "str")
-        elif val is not None and len(val) > 100:
-            raise InvalidLength("label", _max=100)
-        elif val is not None and len(val) < 1:
-            raise InvalidLength("label", _min=0)
-
-        self._json["label"] = str(val)
-
-    @property
-    def color(self) -> int:
-        """
-        The color for the button
-
-        :type: :class:`int`, one of :class:`~ButtonStyles`
-        """
-        return self._json["style"]
-    @color.setter
-    def color(self, val):
-        if ButtonStyles.getColor(val) is None:
-            raise InvalidArgument(str(val) + " is not a valid color")
-        self._json["style"] = ButtonStyles.getColor(val)
-    
-    @property
-    def emoji(self) -> str:
-        """
-        The mention of the emoji before the text
-        
-            .. note::
-                For setting the emoji, you can use a str or discord.Emoji          
-        
-        :type: :class:`str`
-        """
-        if "emoji" not in self._json:
-            return None
-        if "id" not in self._json["emoji"]:
-            return self._json["emoji"]["name"]
-        return f'<{"a" if "animated" in self._json["emoji"] else ""}:{self._json["emoji"]["name"]}:{self._json["emoji"]["id"]}>'
-    @emoji.setter
-    def emoji(self, val: Union[Emoji, str, dict]):
-        if isinstance(val, str):
-            self._json["emoji"] = {
-                "id": None,
-                "name": val
-            }
-        elif isinstance(val, Emoji):
-            self._json["emoji"] = {
-                "id": val.id,
-                "name": val.name,
-                "animated": val.animated
-            }
-        elif isinstance(val, dict):
-            self._json["emoji"] = val
-        else:
-            raise WrongType("emoji", val, ["str", "discord.Emoji", "dict"])
-
-    @property
-    def disabled(self) -> bool:
-        """
-        Whether the button is dissabled
-        If true, the button is shown but not clickable
-
-        :type: :class:`bool`
-        """
-        return self._json["disabled"] if "disabled" in self._json else False
-    @disabled.setter
-    def disabled(self, val):
-        if not isinstance(val, bool):
-            raise WrongType("disabled", val, "bool")
-        self._json["disabled"] = bool(val)
-    
-    @property
-    def hash(self) -> str:
-        """
-        The calculated hash from the discord api for the button
-
-        :type: :class:`str`
-        """
-        return self._json.get('hash', None)
-    # endregion
-    
-    @classmethod
-    def _empty(cls):
-        """Returns an "empty" button"""
-        return cls("empty", "empty")
     @classmethod
     def _fromData(cls, data, new_line=False) -> 'Button':
         """
@@ -561,12 +527,9 @@ class Button(Component):
         Button
             The initialized button
         """
-        b = cls("empty", "empty")
-        b._json = data
-        b.new_line = new_line
-        return b
+        return Button(data["custom_id"], data.get("label"), data["style"], data.get("emoji"), new_line, data.get("disabled", False))
 
-class LinkButton():
+class LinkButton(BaseButton):
     """
     A discord-ui linkbutton
 
@@ -583,7 +546,7 @@ class LinkButton():
     disabled: :class:`bool`, optional
         Whether the button is disabled; default False
     """
-    def __init__(self, url, label="\u200b", emoji=MISSING, new_line=False, disabled=False) -> None:
+    def __init__(self, url, label="\u200b", emoji=None, new_line=False, disabled=False) -> None:
         """
         Creates a new LinkButton object
         
@@ -592,45 +555,14 @@ class LinkButton():
         LinkButton("https://discord.com/", "press me (if you can)!", emoji="😀", disabled=True)
         ```
         """
-        self._json = { "type": ComponentType.BUTTON, "style": ButtonStyles.url }
-
-        self.label = label
+        BaseButton.__init__(self, label, ButtonStyles.url, emoji, new_line, disabled) 
+        self._url = None
         self.url = url
-        self.new_line = new_line
-        self.disabled = disabled
-
-        if emoji is not MISSING:
-            self.emoji = emoji
 
     def __repr__(self) -> str:
         return f"<discord_ui.LinkButton({self.custom_id}:{self.content})>"
-    def __str__(self) -> str:
-        return self.content
-
-    def to_dict(self):
-        return self._json
-
-    # region props
-    @property
-    def component_type(self) -> int:
-        """
-        The message component type
-
-            .. note::
-                The message component type will be always 2 (button)
-        
-        :type: :class:`int`
-        """
-        return self._json["type"]
-
-    @property
-    def content(self) -> str:
-        """
-        The complete content in the button ("{emoji} {label}")
-
-        :type: :class:`str`
-        """
-        return (self.emoji + " " if self.emoji is not None else "") + (self.label or '')
+    def copy(self) -> 'LinkButton':
+        return LinkButton(self.url, self.label, self.emoji, self.new_line, self.disabled)
 
     @property
     def url(self) -> str:
@@ -639,118 +571,36 @@ class LinkButton():
 
         :type: :class:`str`
         """
-        return self._json["url"]
+        return self._url
     @url.setter
     def url(self, val: str):
         if not isinstance(val, str):
             raise WrongType("url", val, "str")
-        self._json["url"] = str(val)
+        self._url = str(val)
 
-    @property
-    def label(self) -> str:
-        """
-        The label displayed on the button
-
-        :type: :class:`str`
-        """
-        return self._json.get("label", None)
-    @label.setter
-    def label(self, val: str):
-        if val is None:
-            val = ""
-        elif val is not None and not isinstance(val, str):
-            raise WrongType("label", val, "str")
-        elif val is not None and len(val) > 80:
-            raise InvalidLength("label", _max=80)
-        elif val is not None and len(val) < 1:
-            raise InvalidArgument("label", _min=0)
-
-        self._json["label"] = str(val)
-
-    @property
-    def color(self) -> int:
-        """
-        The color for the button, will always be ``5`` (link)
-
-        :type: :class:`int`
-        """
-        return self._json["style"]
-    
-    @property
-    def emoji(self) -> str:
-        """
-        The mention of the emoji before the text
-        
-            .. note::
-                For setting the emoji, you can use a str or discord.Emoji          
-        
-        :type: :class:`str`
-        """
-        if "emoji" not in self._json:
-            return None
-        if "id" not in self._json["emoji"]:
-            return self._json["emoji"]["name"]
-        return f'<{"a" if "animated" in self._json["emoji"] else ""}:{self._json["emoji"]["name"]}:{self._json["emoji"]["id"]}>'
-    @emoji.setter
-    def emoji(self, val: Union[Emoji, str, dict]):
-        if isinstance(val, str):
-            self._json["emoji"] = {
-                "id": None,
-                "name": val
-            }
-        elif isinstance(val, Emoji):
-            self._json["emoji"] = {
-                "id": val.id,
-                "name": val.name,
-                "animated": val.animated
-            }
-        elif isinstance(val, dict):
-            self._json["emoji"] = val
-        else:
-            raise WrongType("emoji", val, ["str", "discord.Emoji", "dict"])
-
-    @property
-    def disabled(self) -> bool:
-        """
-        Whether the button is dissabled
-        If true, the button is shown but not clickable
-
-        :type: :class:`bool`
-        """
-        return self._json["disabled"] if "disabled" in self._json else False
-    @disabled.setter
-    def disabled(self, val):
-        if not isinstance(val, bool):
-            raise WrongType("disabled", val, "bool")
-        self._json["disabled"] = bool(val)
-    # endregion
-
-    @classmethod
-    def _empty(cls):
-        """Returns an empty button"""
-        return cls("empty", "empty")
     @classmethod
     def _fromData(cls, data, new_line=False) -> 'LinkButton':
-        """Returns a new Linkbutton initialized from api response data"""
-        b = cls("https://empty", "empty")
-        b._json = data
-        b.new_line = new_line
-        return b
+        return LinkButton(data["url"], data.get("label"), data.get("emoji"), new_line, data.get("disabled", False))
 
 
-class ButtonStyles:
+class ButtonStyles(IntEnum):
     """
     A list of button styles (colors) in message components
     """
-    Primary     =   blurple         = 1
-    Secondary   =   grey            = 2
-    Succes      =   green           = 3
-    Danger      =   red             = 4
-    url                             = 5
+    Primary     =     	 blurple        = 1
+    Secondary   =         grey          = 2
+    Succes      =        green          = 3
+    Danger      =         red           = 4
+    url                                 = 5
+
+    def __str__(self) -> str:
+        return self.name
 
     @classmethod
     def getColor(cls, s):
         if isinstance(s, int):
+            return cls(s)
+        if isinstance(s, cls):
             return s
         s = s.lower()
         if s in ("blurple", "primary"):
@@ -813,20 +663,23 @@ class ActionRow():
         return [x for x in self.items if check(x)]
 
 
-class ComponentType:
+class ComponentType(IntEnum):
     """
     A list of component types
     """
-    ACTION_ROW      =       Action_row       =           1
-    BUTTON          =        Button          =           2
-    SELECT_MENU     =        Select          =           3
+    Action_row      =           1
+    Button          =           2
+    Select          =           3
+
+    def __str__(self) -> str:
+        return self.name
 
 def make_component(data, new_line = False):
-    if data["type"] == ComponentType.BUTTON:
+    if ComponentType(data["type"]) == ComponentType.Button:
         if data["style"] == ButtonStyles.url:
             return LinkButton._fromData(data, new_line)
         return Button._fromData(data, new_line)
-    if data["type"] == ComponentType.SELECT_MENU:
+    if ComponentType(data["type"]) is ComponentType.Select:
         return SelectMenu._fromData(data)
     # if data["type"] == ComponentType.ACTION_ROW:
         # return ActionRow._fromData(data)
\ No newline at end of file
